package org.greenstone.gatherer.file;

import javax.swing.tree.TreePath;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.collection.CollectionManager;
import org.greenstone.gatherer.gui.MirrorPane;
import org.greenstone.gatherer.util.SynchronizedTreeModelTools;


public class WorkspaceTreeModel
    extends FileSystemModel {

    static private FileNode workspace_tree_root = null;
    static private WorkspaceTreeModel workspace_tree_model = null;

    static private FileNode greenstone_collections_node = null;
    static private FileNode local_filespace_node = null;
    static private FileNode downloaded_files_node = null;
    static private FileNode[] folder_shortcuts = null;


    public WorkspaceTreeModel(FileNode root)
    {
	super(root);
    }


    static public WorkspaceTreeModel getWorkspaceTreeModel()
    {
	// Create a root node to contain the various nodes in the workspace tree
 	workspace_tree_root = new FileNode("ABS_ROOT");
	workspace_tree_model = new WorkspaceTreeModel(workspace_tree_root);

	// Add the "Documents in Greenstone Collections" node
	greenstone_collections_node = CollectionManager.getGreenstoneCollectionsMapping();
	workspace_tree_root.insert(greenstone_collections_node);

	// Add the "Local Filespace" node
	local_filespace_node = FileSystem.getLocalFilespaceNode(workspace_tree_model);
	workspace_tree_root.insert(local_filespace_node);

	// Add the "Home Folder" node
	workspace_tree_root.insert(FileSystem.getHomeFolderNode());

	// Add the "Downloaded Files" node, if Mirroring is enabled
	if (Gatherer.g_man.mirror_pane != null) {
	    downloaded_files_node = FileSystem.getDownloadedFilesNode();
	    workspace_tree_root.insert(downloaded_files_node);
	}

	// Add any folder shortcuts the user has created
	refreshFolderShortcuts();

 	return workspace_tree_model;
    }


    static public void refreshGreenstoneCollectionsNode()
    {
	greenstone_collections_node.refresh();
    }


    static public void refreshDownloadedFilesNode()
    {
	downloaded_files_node.refresh();
    }


    static public void refreshFolderShortcuts()
    {
	// Check for new/deleted folder shortcuts
	FileNode[] old_folder_shortcuts = folder_shortcuts;
	folder_shortcuts = CollectionManager.getFolderShortcuts();

	// Remove any deleted shortcuts from the tree
	if (old_folder_shortcuts != null) {
	    for (int i = 0; i < old_folder_shortcuts.length; i++) {
		if (!doesArrayContain(folder_shortcuts, old_folder_shortcuts[i])) {
		    DebugStream.println("Deleted shortcut: " + old_folder_shortcuts[i]);
		    SynchronizedTreeModelTools.removeNodeFromParent(workspace_tree_model,
								    old_folder_shortcuts[i]);
		}
	    }
	}

	// Add any new shortcuts to the tree
	if (folder_shortcuts != null) {
	    for (int i = 0; i < folder_shortcuts.length; i++) {
		if (!doesArrayContain(old_folder_shortcuts, folder_shortcuts[i])) {
		    DebugStream.println("Added shortcut: " + folder_shortcuts[i]);
		    SynchronizedTreeModelTools.insertNodeInto(workspace_tree_model, workspace_tree_root, folder_shortcuts[i], false);
		}
	    }
	}
    }


    // -- This code is necessary to support the workspace tree file filter --
    public void refresh(TreePath path)
    {
	// If we're not refreshing the whole tree just refresh a certain path
	if (path != null) {
	    super.refresh(path);
	    return;
	}

	// Refresh each of the nodes in the workspace tree
	for (int i = 0; i < workspace_tree_root.getChildCount(); i++) {
	    FileNode child_node = (FileNode) workspace_tree_root.getChildAt(i);
	    super.refresh(new TreePath(child_node.getPath()));
	}

	// Refresh the local filespace tree specially (it is not unmapped so not refreshed)
	for (int i = 0; i < local_filespace_node.getChildCount(); i++) {
	    FileNode child_node = (FileNode) local_filespace_node.getChildAt(i);
	    super.refresh(new TreePath(child_node.getPath()));
	}
    }


    static private boolean doesArrayContain(Object[] array, Object item)
    {
	if (array == null) {
	    return false;
	}

	for (int i = 0; i < array.length; i++) {
	    if (item.toString().equals(array[i].toString())) {
		return true;
	    }
	}

	return false;
    }
}
