/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Project, NZDL, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 2003 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;

// this has been changed to not offer the choice of a new metadata set, cos I haven't made this work properly.
/** Prompt used when opening a non-gatherer built collection. We tell the user that they need to select metadata sets, and (in future) give them a choice between selecting an existing set or creating a new one */
public class ExternalCollectionPrompt
    extends ModalDialog {

    public static int ERROR = 0;
    public static int NEW_META_SET = 1;
    public static int EXISTING_META_SET = 2;

    private boolean cancelled = true;
    private JDialog self;
    private JRadioButton new_mds_button;
    private JRadioButton existing_mds_button;
    

    static private Dimension size = new Dimension(600, 250);

    public ExternalCollectionPrompt() {
	super(Gatherer.g_man, true);
	this.self = this;
	setJMenuBar(new SimpleMenuBar("loadinganexternalcollection"));
	setModal(true);
	setSize(size);
	Dictionary.setText(this, "ExternalCollectionPrompt.Title");

	// Creation
	JPanel content_pane = (JPanel) getContentPane();
	JPanel center_pane = new JPanel();

	JPanel instructions_panel = new JPanel();
	JTextArea instructions_textarea;
	instructions_textarea = new JTextArea();
	instructions_textarea.setCaretPosition(0);
	instructions_textarea.setEditable(false);
	instructions_textarea.setLineWrap(true);
	instructions_textarea.setRows(6);
	instructions_textarea.setWrapStyleWord(true);
	Dictionary.registerText(instructions_textarea, "ExternalCollectionPrompt.Instructions_1"); // change to instructions_2 when add the other choice back in

	/*	new_mds_button = new JRadioButton();
	new_mds_button.setBackground(Configuration.getColor("coloring.collection_tree_background", false));
	//new_mds_button.setMnemonic(KeyEvent.VK_I);
	new_mds_button.setOpaque(false);
	Dictionary.registerText(new_mds_button, "ExternalCollectionPrompt.NewMDS");
	existing_mds_button = new JRadioButton();
	existing_mds_button.setBackground(Configuration.getColor("coloring.collection_tree_background", false));
	//existing_mds_button.setMnemonic(KeyEvent.VK_X);
	existing_mds_button.setOpaque(false);
	Dictionary.registerText(existing_mds_button, "ExternalCollectionPrompt.ExistingMDS");

	ButtonGroup bg = new ButtonGroup();
	bg.add(new_mds_button);
	bg.add(existing_mds_button);
	new_mds_button.setSelected(true);

	JPanel selection_pane = new JPanel();
	selection_pane.setLayout(new GridLayout(2,1));
	selection_pane.add(new_mds_button);
	selection_pane.add(existing_mds_button);
	*/
	JPanel button_pane = new JPanel();
	JButton ok_button = new GLIButton();
	ok_button.setMnemonic(KeyEvent.VK_O);
	Dictionary.setBoth(ok_button, "General.OK", "General.OK_Tooltip");
	JButton cancel_button = new GLIButton();
	cancel_button.setMnemonic(KeyEvent.VK_C);
	Dictionary.setBoth(cancel_button, "General.Cancel", "General.Pure_Cancel_Tooltip");

	// Connection
	ok_button.addActionListener(new OKButtonListener());
	cancel_button.addActionListener(new CancelButtonListener());
	//sets_list.addListSelectionListener(new MetadataListSelectionListener());

	// Display
	instructions_panel.setLayout(new GridLayout(1,1));
	instructions_panel.add(instructions_textarea);


	//center_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	//center_pane.setLayout(new GridLayout(2,1,0,5));
	//center_pane.add(selection_pane);
	
	button_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	button_pane.setLayout(new GridLayout(1,2,5,0));
	button_pane.add(ok_button);
	button_pane.add(cancel_button);

	content_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	content_pane.setLayout(new BorderLayout());
	content_pane.add(instructions_panel, BorderLayout.NORTH);
	//content_pane.add(center_pane, BorderLayout.CENTER);
	content_pane.add(button_pane, BorderLayout.SOUTH);

	// Show
	Dimension screen_size = Configuration.screen_size;
	setLocation((screen_size.width - size.width) / 2, (screen_size.height - size.height) / 2);
	setVisible(true);
    }

    public int getMetadataChoice() {
	return NEW_META_SET;
	
	/*if (existing_mds_button.isSelected()) {
	    return EXISTING_META_SET;
	}
	if (new_mds_button.isSelected()) {
	    return NEW_META_SET;
	}
	// something wrong
	return ERROR;*/
    }
    public boolean isCancelled() {
	return cancelled;
    }
    private class CancelButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    cancelled = true;
	    self.dispose();
	}
    }
    public class OKButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    cancelled = false;
	    self.dispose();
		
	}
    }
}
